<?php
/*
Plugin Name: TCB Pay
Plugin URI: 
Description: WooCommerce with TCB Pay payment gateway.
Version: 1.0.1
Author: TCB Pay
Author URI: https://www.tcbpay.com

Copyright: 2019 TCB Pay
License: GNU General Public License v3.0
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/
    if ( ! defined( 'ABSPATH' ) )
        exit;
    add_action('plugins_loaded', 'woocommerce_tcbpay_init', 0);

    function woocommerce_tcbpay_init() {

        if ( !class_exists( 'WC_Payment_Gateway' ) ) return;

    /**
     * Gateway class
     */
    class WC_Tcbpay extends WC_Payment_Gateway {
        public $api_login;
        public $trans_key;
        public $secret_key;
        public $environment;
		
        // Setup our Gateway's id, description and other values
        function __construct() {

            // The global ID for this Payment method
            $this->id = "tcbpay";

            // The Title shown on the top of the Payment Gateways Page next to all the other Payment Gateways
            $this->method_title = __( "TCBPay", 'tcbpay' );

            // The description for this Payment Gateway, shown on the actual Payment options page on the backend
            $this->method_description = __( "This controls the title which the user sees during checkout.", 'tcbpay' );

            // The title to be used for the vertical tabs that can be ordered top to bottom
            $this->title = __( "TCBPay", 'tcbpay' );

            // If you want to show an image next to the gateway's name on the frontend, enter a URL to an image.
            $this->icon =  plugins_url( 'images/logo.png' , __FILE__ );

            // Bool. Can be set to true if you want payment fields to show on the checkout 
            // if doing a direct integration, which we are doing in this case
            $this->has_fields = true;

            // Supports the default credit card form
            $this->supports = array( 'default_credit_card_form' );

            // This basically defines your settings which are then loaded with init_settings()
            $this->init_form_fields();

            // After init_settings() is called, you can get the settings and load them into variables, e.g:
            // $this->title = $this->get_option( 'title' );
            $this->init_settings();
            
            // Turn these settings into variables we can use
            foreach ( $this->settings as $setting_key => $value ) {
                $this->$setting_key = $value;
            }
            
            // Lets check for SSL
            add_action( 'admin_notices', array( $this,  'do_ssl_check' ) );
            
            // Save settings
            if ( is_admin() ) {
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
            }       
        } // End __construct()

        // Build the administration fields for this specific Gateway
        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title'     => __( 'Enable / Disable', 'tcbpay' ),
                    'label'     => __( 'Enable this payment gateway', 'tcbpay' ),
                    'type'      => 'checkbox',
                    'default'   => 'no',
                ),
                'title' => array(
                    'title'     => __( 'Title', 'tcbpay' ),
                    'type'      => 'text',
                    'desc_tip'  => __( 'Payment title the customer will see during the checkout process.', 'tcbpay' ),
                    'default'   => __( 'Credit card', 'tcbpay' ),
                ),
                'description' => array(
                    'title'     => __( 'Description', 'tcbpay' ),
                    'type'      => 'textarea',
                    'desc_tip'  => __( 'Payment description the customer will see during the checkout process.', 'tcbpay' ),
                    'default'   => __( 'Pay securely using your credit card.', 'tcbpay' ),
                    'css'       => 'max-width:350px;'
                ),
                'api_login' => array(
                    'title'     => __( 'Username', 'tcbpay' ),
                    'type'      => 'text',
                    'desc_tip'  => __( 'This is the username provided by Tcbpay when you signed up for an account.', 'tcbpay' ),
                ),
                'trans_key' => array(
                    'title'     => __( 'API Key', 'tcbpay' ),
                    'type'      => 'text',
                    'desc_tip'  => __( 'This is the API Key provided by Tcbpay when you signed up for an account.', 'tcbpay' ),
                ),
                'secret_key' => array(
                    'title'     => __( 'Secret Key', 'tcbpay' ),
                    'type'      => 'text',
                    'desc_tip'  => __( 'This is the Secret Key provided by Tcbpay when you signed up for an account.', 'tcbpay' ),
                ),
                'environment' => array(
                    'title'     => __( 'Tcbpay Test Mode', 'tcbpay' ),
                    'label'     => __( 'Enable Test Mode', 'tcbpay' ),
                    'type'      => 'checkbox',
                    'description' => __( 'Place the payment gateway in test mode.', 'tcbpay' ),
                    'default'   => 'no',
                )
            );      
        }
        
        // Submit payment and handle response
        public function process_payment( $order_id ) {
            global $woocommerce;
            
            
            $customer_order = new WC_Order( $order_id );
            
            
            $environment = ( $this->environment == "yes" ) ? 'TRUE' : 'FALSE';
            
            $environment_url = ( "FALSE" == $environment ) 
                               ? 'https://gateway2.tcbpay.com/mobile_plugin/sale.php'
                               : 'https://gateway2-stg.tcbpay.com/mobile_plugin/sale.php';

            $card_expairy = explode('/', $_POST['tcbpay-card-expiry']);
            
            $payload = array(
                "username"              => $this->api_login,
                "api_key"               => $this->trans_key,
                "amount"                => $customer_order->order_total,
                "creditcard_number"     => $_POST['tcbpay_enc_card_number'],
                "cvv"                   => ( isset( $_POST['tcbpay-card-cvc'] ) ) ? $_POST['tcbpay-card-cvc'] : '',
                "expire_month"          => trim($card_expairy[0]),
                "expire_year"           => trim($card_expairy[1]),
                "first_name"            => $customer_order->billing_first_name,
                "last_name"             => $customer_order->billing_last_name,
                "address"               => $customer_order->billing_address_1,
                "city"                  => $customer_order->billing_city,
                "state"                 => $customer_order->billing_state,
                "zip_code"              => $customer_order->billing_postcode,
                "country_code"          => $customer_order->billing_country,
                "phone_number"          => $customer_order->billing_phone,
                "email"                 => $customer_order->billing_email,
                "currency"              => get_woocommerce_currency(),
				"client_ip"				=> $_SERVER['REMOTE_ADDR'],
                "source"                => "API_WOOCOMMERCE"
            );

            $gmtime = strtotime(gmdate('Y-m-d H:i:s'));

            $response = wp_remote_post( $environment_url, array(
                'method'    => 'POST',
                'body'      => http_build_query( $payload ),
                'timeout'   => 90,
                'sslverify' => false,
                'headers' => array(
                    'hash_timestamp' => $gmtime,
                    'hash' => md5($this->api_login."|".$this->trans_key."|".$this->secret_key."|".$gmtime)
                ),
            ) );

            if ( is_wp_error( $response ) ) 
                throw new Exception( __( 'We are currently experiencing problems trying to connect to this payment gateway. Sorry for the inconvenience.', 'tcbpay' ) );

            if ( empty( $response['body'] ) )
                throw new Exception( __( 'Tcbpay\'s Response was empty.', 'tcbpay' ) );
                
            // Retrieve the body's resopnse if no errors found
            $response_body = wp_remote_retrieve_body( $response );
            $response_body = json_decode($response_body);


            // Test the code to know if the transaction went through or not.
            // 1 or 4 means the transaction was a success
            if ( $response_body->response->code == '00' ) {
                // Payment has been successful
                $customer_order->add_order_note( __( 'Tcbpay payment completed. Transaction id: #'.$response_body->response->transaction_id, 'tcbpay' ) );
                                                     
                // Mark order as Paid
                $customer_order->payment_complete("Transaction id: #".$response_body->response->transaction_id);

                // Empty the cart (Very important step)
                $woocommerce->cart->empty_cart();
		        
                // Redirect to thank you page
                return array(
                    'result'   => 'success',
                    'redirect' => $this->get_return_url( $customer_order ),
                );
            } else {
                // Transaction was not succesful
                // Add notice to the cart
                wc_add_notice( $response_body->response->description, 'error' );
                // Add note to the order for your reference
                $customer_order->add_order_note( 'Error: '. $response_body->response->description );
            }

        }
        
        // Validate fields
        public function validate_fields() {
            return true;
        }
        
        // Check if we are forcing SSL on checkout pages
        // Custom function not required by the Gateway
        public function do_ssl_check() {
            if( $this->enabled == "yes" ) {
                if( get_option( 'woocommerce_force_ssl_checkout' ) == "no" ) {
                    echo "<div class=\"error\"><p>". sprintf( __( "<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href=\"%s\">forcing the checkout pages to be secured.</a>" ), $this->method_title, admin_url( 'admin.php?page=wc-settings&tab=checkout' ) ) ."</p></div>";  
                }
            }       
        }


    }

    /**
     * Add the Gateway to WooCommerce
     **/
    function woocommerce_add_tcbpay_gateway($methods) {
        $methods[] = 'WC_TCBPay';
		
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'woocommerce_add_tcbpay_gateway' );

    function tcbpay_api_script() {

            $gatewayInstance = new WC_TCBPay();

            $jsPath = $gatewayInstance->environment == "yes" ? 'https://gateway2-stg.tcbpay.com/services/proxynization_api.js' : 'https://gateway2.tcbpay.com/services/proxynization_api.js';

            wp_register_script('tacbpay-gateway', $jsPath, array('jquery'), null, true );
            wp_enqueue_script('tacbpay-gateway');
            wp_register_script('tacbpay-gateway-callback', plugin_dir_url(__FILE__) . 'tcbpay-callback.js', array('jquery'), null, true );
            wp_enqueue_script('tacbpay-gateway-callback');

    }
    add_action( 'wp_enqueue_scripts', 'tcbpay_api_script', 20, 1);

}




?>